@@ -24,15 +24,21 @@ use Text::Tabs;
 # Throughout the files read by this script are lines like
 #
 # =for apidoc_section Section Name
+# =for apidoc_section $section_name_variable
 #
 # "Section Name" (after having been stripped of leading space) must be one of
-# the legal section names, or an error is thrown.  All API elements defined
-# between this line and the next 'apidoc_section' line will go into the
-# section "Section Name", sorted by dictionary order within it.  perlintern
-# and perlapi are parallel documents, each potentially with a section "Section
-# Name".  Each element is marked as to which document it goes into.  If there
-# are none for a particular section in perlapi, that section is
-# omitted.
+# the legal section names, or an error is thrown.  $section_name_variable must
+# be one of the legal section name variables defined below; these expand to
+# legal section names.  This form is used so that minor wording changes in
+# these titles can be confied to this file.  All the names of the variables
+# end in '_scn'; this suffix is optional in the apidoc_section lines.
+#
+# All API elements defined between this line and the next 'apidoc_section'
+# line will go into the section "Section Name" (or $section_name_variable),
+# sorted by dictionary order within it.  perlintern and perlapi are parallel
+# documents, each potentially with a section "Section Name".  Each element is
+# marked as to which document it goes into.  If there are none for a
+# particular section in perlapi, that section is omitted.
 #
 # Also, in .[ch] files, there may be
 #
@@ -44,7 +50,7 @@ use Text::Tabs;
 # are used as a heading for section "Section Name" (in both perlintern and
 # perlapi).  This includes any =head[2-5].  If more than one '=head1 Section
 # Name' line has content, they appear in the generated pod in an undefined
-# order.
+# order.  Note that you can't use a $section_name_variable in =head1 lines
 #
 # The next =head1, =for apidoc_section, or file end terminates what goes into
 # the current section
@@ -60,6 +66,10 @@ use Text::Tabs;
 use strict;
 use warnings;
 
+# 80 column terminal - 2 for pager adding 2 columns; -4 for indent for
+# non-heading lines;
+my $max_width = 80 - 2 - 4;
+
 if (@ARGV) {
     my $workdir = shift;
     chdir $workdir
@@ -88,36 +98,106 @@ my $link_text = "Described in";
 my $description_indent = 4;
 my $usage_indent = 3;   # + initial blank yields 4 total
 
+my $AV_scn = 'AV Handling';
+my $callback_scn = 'Callback Functions';
+my $casting_scn = 'Casting';
+my $casing_scn = 'Character case changing';
+my $classification_scn = 'Character classification';
+my $names_scn = 'Character names';
+my $scope_scn = 'Compile-time scope hooks';
+my $compiler_scn = 'Compiler and Preprocessor information';
+my $directives_scn = 'Compiler directives';
+my $concurrency_scn = 'Concurrency';
+my $COP_scn = 'COP Hint Hashes';
+my $CV_scn = 'CV Handling';
+my $custom_scn = 'Custom Operators';
+my $debugging_scn = 'Debugging';
+my $display_scn = 'Display functions';
+my $embedding_scn = 'Embedding and Interpreter Cloning';
+my $errno_scn = 'Errno';
+my $exceptions_scn = 'Exception Handling (simple) Macros';
+my $filesystem_scn = 'Filesystem configuration values';
+my $floating_scn = 'Floating point configuration values';
+my $formats_scn = 'Formats';
+my $genconfig_scn = 'General Configuration';
+my $globals_scn = 'Global Variables';
+my $GV_scn = 'GV Handling';
+my $hook_scn = 'Hook manipulation';
+my $HV_scn = 'HV Handling';
+my $io_scn = 'Input/Output';
+my $integer_scn = 'Integer configuration values';
+my $lexer_scn = 'Lexer interface';
+my $locale_scn = 'Locales';
+my $magic_scn = 'Magic';
+my $memory_scn = 'Memory Management';
+my $MRO_scn = 'MRO';
+my $multicall_scn = 'Multicall Functions';
+my $numeric_scn = 'Numeric Functions';
+my $optree_construction_scn = 'Optree construction';
+my $optree_manipulation_scn = 'Optree Manipulation Functions';
+my $pack_scn = 'Pack and Unpack';
+my $pad_scn = 'Pad Data Structures';
+my $password_scn = 'Password and Group access';
+my $paths_scn = 'Paths to system commands';
+my $prototypes_scn = 'Prototype information';
+my $regexp_scn = 'REGEXP Functions';
+my $signals_scn = 'Signals';
+my $site_scn = 'Site configuration';
+my $sockets_scn = 'Sockets configuration values';
+my $filters_scn = 'Source Filters';
+my $stack_scn = 'Stack Manipulation Macros';
+my $string_scn = 'String Handling';
+my $SV_flags_scn = 'SV Flags';
+my $SV_scn = 'SV Handling';
+my $time_scn = 'Time';
+my $typedefs_scn = 'Typedef names';
+my $unicode_scn = 'Unicode Support';
+my $utility_scn = 'Utility Functions';
+my $versioning_scn = 'Versioning';
+my $warning_scn = 'Warning and Dieing';
+my $XS_scn = 'XS';
+
+# Kept separate at end
+my $undocumented_scn = 'Undocumented elements';
+
 my %valid_sections = (
-    'AV Handling' => {},
-    'Callback Functions' => {},
-    'Casting' => {},
-    'Character case changing' => {},
-    'Character classification' => {},
-    'Compile-time scope hooks' => {},
-    'Compiler and Preprocessor information' => {},
-    'Compiler directives' => {},
-    'Concurrency' => {},
-    'COP Hint Hashes' => {},
-    'CV Handling' => {},
-    'Custom Operators' => {},
-    'Display and Dump functions' => {},
-    'Embedding and Interpreter Cloning' => {},
-    'Errno' => {},
-    'Exception Handling (simple) Macros' => {},
-    'Filesystem configuration values' => {
+    $AV_scn => {},
+    $callback_scn => {},
+    $casting_scn => {},
+    $casing_scn => {},
+    $classification_scn => {},
+    $scope_scn => {},
+    $compiler_scn => {},
+    $directives_scn => {},
+    $concurrency_scn => {},
+    $COP_scn => {},
+    $CV_scn => {
+        header => <<~'EOT',
+            This section documents functions to manipulate CVs which are
+            code-values, meaning subroutines.  For more information, see
+            L<perlguts>.
+            EOT
+    },
+
+    $custom_scn => {},
+    $debugging_scn => {},
+    $display_scn => {},
+    $embedding_scn => {},
+    $errno_scn => {},
+    $exceptions_scn => {},
+    $filesystem_scn => {
         header => <<~'EOT',
             Also see L</List of capability HAS_foo symbols>.
             EOT
         },
-    'Floating point configuration values' => {
+    $floating_scn => {
         header => <<~'EOT',
             Also L</List of capability HAS_foo symbols> lists capabilities
             that arent in this section.  For example C<HAS_ASINH>, for the
             hyperbolic sine function.
             EOT
         },
-    'Formats' => {
+    $formats_scn => {
         header => <<~'EOT',
             These are used for formatting the corresponding type For example,
             instead of saying
@@ -132,7 +212,7 @@ my %valid_sections = (
             printed as C<%d>, C<%ld>, or something else.
             EOT
       },
-    'General Configuration' => {
+    $genconfig_scn => {
         header => <<~'EOT',
             This section contains configuration information not otherwise
             found in the more specialized sections of this document.  At the
@@ -201,53 +281,62 @@ my %valid_sections = (
             =back
             EOT
       },
-    'Global Variables' => {},
-    'GV Handling' => {},
-    'Hook manipulation' => {},
-    'HV Handling' => {},
-    'Input/Output' => {},
-    'Integer configuration values' => {},
-    'Lexer interface' => {},
-    'Locales' => {},
-    'Magic' => {},
-    'Memory Management' => {},
-    'MRO' => {},
-    'Multicall Functions' => {},
-    'Numeric Functions' => {},
-    'Optree construction' => {},
-    'Optree Manipulation Functions' => {},
-    'Pack and Unpack' => {},
-    'Pad Data Structures' => {},
-    'Password and Group access' => {},
-    'Paths to system commands' => {},
-    'Per-Interpreter Variables' => {},
-    'Prototype information' => {},
-    'REGEXP Functions' => {},
-    'Signals' => {},
-    'Site configuration' => {
+    $globals_scn => {},
+    $GV_scn => {},
+    $hook_scn => {},
+    $HV_scn => {},
+    $io_scn => {},
+    $integer_scn => {},
+    $lexer_scn => {},
+    $locale_scn => {},
+    $magic_scn => {},
+    $memory_scn => {},
+    $MRO_scn => {},
+    $multicall_scn => {},
+    $numeric_scn => {},
+    $optree_construction_scn => {},
+    $optree_manipulation_scn => {},
+    $pack_scn => {},
+    $pad_scn => {},
+    $password_scn => {},
+    $paths_scn => {},
+    $prototypes_scn => {},
+    $regexp_scn => {},
+    $signals_scn => {},
+    $site_scn => {
         header => <<~'EOT',
             These variables give details as to where various libraries,
             installation destinations, I<etc.>, go, as well as what various
             installation options were selected
             EOT
       },
-    'Sockets configuration values' => {},
-    'Source Filters' => {},
-    'Stack Manipulation Macros' => {},
-    'String Handling' => {
-        header => <<~'EOT',
-            See also C<L</Unicode Support>>.
+    $sockets_scn => {},
+    $filters_scn => {},
+    $stack_scn => {},
+    $string_scn => {
+        header => <<~EOT,
+            See also C<L</$unicode_scn>>.
             EOT
       },
-    'SV Flags' => {},
-    'SV Handling' => {},
-    'Time' => {},
-    'Typedef names' => {},
-    'Unicode Support' => {},
-    'Utility Functions' => {},
-    'Versioning' => {},
-    'Warning and Dieing' => {},
-    'XS' => {},
+    $SV_flags_scn => {},
+    $SV_scn => {},
+    $time_scn => {},
+    $typedefs_scn => {},
+    $unicode_scn => {
+        header => <<~EOT,
+            L<perlguts/Unicode Support> has an introduction to this API.
+
+            See also C<L</$classification_scn>>,
+            C<L</$casing_scn>>,
+            and C<L</$string_scn>>.
+            Various functions outside this section also work specially with
+            Unicode.  Search for the string "utf8" in this document.
+            EOT
+      },
+    $utility_scn => {},
+    $versioning_scn => {},
+    $warning_scn => {},
+    $XS_scn => {},
 );
 
 # Somewhat loose match for an apidoc line so we can catch minor typos.
@@ -291,7 +380,7 @@ Expected:
 (or 'apidoc_item')
 EOS
 
-    die "Only [$display_flags] allowed in apidoc_item"
+    die "Only [$display_flags] allowed in apidoc_item:\n$in"
                             if $is_item && $flags =~ /[^$display_flags]/;
 
     return ($name, $flags, $ret_type, $is_item, $proto_in_file, @args);
@@ -313,10 +402,49 @@ sub embed_override($) {
     return ($flags, $embed_docref->{'ret_type'}, $embed_docref->{args}->@*);
 }
 
+# The section that is in effect at the beginning of the given file.  If not
+# listed here, an apidoc_section line must precede any apidoc lines.
+# This allows the files listed here that generally are single-purpose, to not
+# have to worry about the autodoc section
+my %initial_file_section = (
+                            'av.c' => $AV_scn,
+                            'av.h' => $AV_scn,
+                            'cv.h' => $CV_scn,
+                            'doio.c' => $io_scn,
+                            'gv.c' => $GV_scn,
+                            'gv.h' => $GV_scn,
+                            'hv.h' => $HV_scn,
+                            'locale.c' => $locale_scn,
+                            'malloc.c' => $memory_scn,
+                            'numeric.c' => $numeric_scn,
+                            'opnames.h' => $optree_construction_scn,
+                            'pad.h'=> $pad_scn,
+                            'patchlevel.h' => $versioning_scn,
+                            'perlio.h' => $io_scn,
+                            'pod/perlapio.pod' => $io_scn,
+                            'pod/perlcall.pod' => $callback_scn,
+                            'pod/perlembed.pod' => $embedding_scn,
+                            'pod/perlfilter.pod' => $filters_scn,
+                            'pod/perliol.pod' => $io_scn,
+                            'pod/perlmroapi.pod' => $MRO_scn,
+                            'pod/perlreguts.pod' => $regexp_scn,
+                            'pp_pack.c' => $pack_scn,
+                            'pp_sort.c' => $SV_scn,
+                            'regcomp.c' => $regexp_scn,
+                            'regexp.h' => $regexp_scn,
+                            'unicode_constants.h' => $unicode_scn,
+                            'utf8.c' => $unicode_scn,
+                            'utf8.h' => $unicode_scn,
+                            'vutil.c' => $versioning_scn,
+                           );
+
 sub autodoc ($$) { # parse a file and extract documentation info
     my($fh,$file) = @_;
     my($in, $line_num, $header, $section);
 
+    $section = $initial_file_section{$file}
+                                    if defined $initial_file_section{$file};
+
     my $file_is_C = $file =~ / \. [ch] $ /x;
 
     # Count lines easier
@@ -341,6 +469,11 @@ sub autodoc ($$) { # parse a file and extract documentation info
         if ($in=~ /^ = (?: for [ ]+ apidoc_section | head1 ) [ ]+ (.*) /x) {
 
             $section = $1;
+            if ($section =~ / ^ \$ /x) {
+                $section .= '_scn' unless $section =~ / _scn $ /;
+                $section = eval "$section";
+                die "Unknown \$section variable '$section' in $file: $@" if $@;
+            }
             die "Unknown section name '$section' in $file near line $.\n"
                                     unless defined $valid_sections{$section};
 
@@ -370,8 +503,8 @@ sub autodoc ($$) { # parse a file and extract documentation info
                 $missing{$element_name} = $file;
             }
 
-            die "flag $1 is not legal (for function $element_name (from $file))"
-                        if $flags =~ / ( [^AabCDdEeFfhiMmNnTOoPpRrSsUuWXxy] ) /x;
+            die "flag '$1' is not legal (for function $element_name (from $file))"
+                        if $flags =~ / ( [^AabCDdEeFfGhiMmNnTOoPpRrSsUuWXxy] ) /x;
 
             die "'u' flag must also have 'm' or 'y' flags' for $element_name"
                                             if $flags =~ /u/ && $flags !~ /[my]/;
@@ -428,7 +561,8 @@ sub autodoc ($$) { # parse a file and extract documentation info
                                                         unless $element_name;
 
             # We accept blank lines between these, but nothing else;
-            die "apidoc_item lines must immediately follow apidoc lines"
+            die "apidoc_item lines must immediately follow apidoc lines for "
+              . " '$element_name' in $file"
                                                             if $text =~ /\S/;
             # Override this line with any info in embed.fnc
             my ($embed_flags, $embed_ret_type, @embed_args)
@@ -489,7 +623,7 @@ sub autodoc ($$) { # parse a file and extract documentation info
             my $is_link_only = ($flags =~ /h/);
             if ($is_link_only) {
                 if ($file_is_C) {
-                    die "Can't currently handle link with items to it" if @items;
+                    die "Can't currently handle link with items to it:\n$in" if @items;
                     redo;    # Don't put anything if C source
                 }
 
@@ -535,6 +669,7 @@ my %configs;
 my @has_defs;
 my @has_r_defs;     # Reentrant symbols
 my @include_defs;
+
 sub parse_config_h {
     use re '/aa';   # Everthing is ASCII in this file
 
@@ -854,11 +989,16 @@ sub parse_config_h {
             my $dash_or_spaces = qr/ - | \s+ /x;
             my $pod = $configs{$name}{pod};
             if ($name =~ / ^ USE_ /x) {
-                $configs{$name}{'section'} = 'Site configuration';
+                $configs{$name}{'section'} = $site_scn;
             }
             elsif ($name =~ / SLEEP | (*nlb:SYS_) TIME | TZ | $sb TM $sb /x)
             {
-                $configs{$name}{'section'} = 'Time';
+                $configs{$name}{'section'} = $time_scn;
+            }
+            elsif (   $name =~ / ^ [[:alpha:]]+ f $ /x
+                   && $configs{$name}{pod} =~ m/ \b format \b /ix)
+            {
+                $configs{$name}{'section'} = $formats_scn;
             }
             elsif ($name =~ /  DOUBLE | FLOAT | LONGDBL | LDBL | ^ NV
                             | $sb CASTFLAGS $sb
@@ -868,35 +1008,35 @@ sub parse_config_h {
                             /x)
             {
                 $configs{$name}{'section'} =
-                                    'Floating point configuration values';
+                                    $floating_scn;
             }
             elsif ($name =~ / (?: POS | OFF | DIR ) 64 /x) {
-                $configs{$name}{'section'} = 'Filesystem configuration values';
+                $configs{$name}{'section'} = $filesystem_scn;
             }
             elsif (   $name =~ / $sb (?: BUILTIN | CPP ) $sb | ^ CPP /x
                    || $configs{$name}{pod} =~ m/ \b align /x)
             {
-                $configs{$name}{'section'} = 'Compiler and Preprocessor information';
+                $configs{$name}{'section'} = $compiler_scn;
             }
             elsif ($name =~ / ^ [IU] [ \d V ]
                             | ^ INT | SHORT | LONG | QUAD | 64 | 32 /xx)
             {
-                $configs{$name}{'section'} = 'Integer configuration values';
+                $configs{$name}{'section'} = $integer_scn;
             }
             elsif ($name =~ / $sb t $sb /x) {
-                $configs{$name}{'section'} = 'Typedef names';
+                $configs{$name}{'section'} = $typedefs_scn;
                 $flags .= 'y';
             }
             elsif (   $name =~ / ^ PERL_ ( PRI | SCN ) | $sb FORMAT $sb /x
                     && $configs{$name}{pod} =~ m/ \b format \b /ix)
             {
-                $configs{$name}{'section'} = 'Formats';
+                $configs{$name}{'section'} = $formats_scn;
             }
             elsif ($name =~ / BACKTRACE /x) {
-                $configs{$name}{'section'} = 'Display and Dump functions';
+                $configs{$name}{'section'} = $debugging_scn;
             }
             elsif ($name =~ / ALLOC $sb /x) {
-                $configs{$name}{'section'} = 'Memory Management';
+                $configs{$name}{'section'} = $memory_scn;
             }
             elsif (   $name =~ /   STDIO | FCNTL | EOF | FFLUSH
                                 | $sb FILE $sb
@@ -916,50 +1056,50 @@ sub parse_config_h {
                                                 | statfs
                                                 !x)
             {
-                $configs{$name}{'section'} = 'Filesystem configuration values';
+                $configs{$name}{'section'} = $filesystem_scn;
             }
             elsif ($name =~ / ^ SIG | SIGINFO | signal /ix) {
-                $configs{$name}{'section'} = 'Signals';
+                $configs{$name}{'section'} = $signals_scn;
             }
             elsif ($name =~ / $sb ( PROTO (?: TYPE)? S? ) $sb /x) {
-                $configs{$name}{'section'} = 'Prototype information';
+                $configs{$name}{'section'} = $prototypes_scn;
             }
             elsif (   $name =~ / ^ LOC_ /x
                     || $configs{$name}{pod} =~ /full path/i)
             {
-                $configs{$name}{'section'} = 'Paths to system commands';
+                $configs{$name}{'section'} = $paths_scn;
             }
             elsif ($name =~ / $sb LC_ | LOCALE | langinfo /xi) {
-                $configs{$name}{'section'} = 'Locales';
+                $configs{$name}{'section'} = $locale_scn;
             }
             elsif ($configs{$name}{pod} =~ /  GCC | C99 | C\+\+ /xi) {
-                $configs{$name}{'section'} = 'Compiler and Preprocessor information';
+                $configs{$name}{'section'} = $compiler_scn;
             }
             elsif ($name =~ / PASSW (OR)? D | ^ PW | ( PW | GR ) ENT /x)
             {
-                $configs{$name}{'section'} = 'Password and Group access';
+                $configs{$name}{'section'} = $password_scn;
             }
             elsif ($name =~ /  SOCKET | $sb SOCK /x) {
-                $configs{$name}{'section'} = 'Sockets configuration values';
+                $configs{$name}{'section'} = $sockets_scn;
             }
             elsif (   $name =~ / THREAD | MULTIPLICITY /x
                     || $configs{$name}{pod} =~ m/ \b pthread /ix)
             {
-                $configs{$name}{'section'} = 'Concurrency';
+                $configs{$name}{'section'} = $concurrency_scn;
             }
             elsif ($name =~ /  PERL | ^ PRIV | SITE | ARCH | BIN
                                 | VENDOR | ^ USE
                             /x)
             {
-                $configs{$name}{'section'} = 'Site configuration';
+                $configs{$name}{'section'} = $site_scn;
             }
             elsif (   $pod =~ / \b floating $dash_or_spaces point \b /ix
-                    || $pod =~ / \b (double | single) $dash_or_spaces precision \b /ix
-                    || $pod =~ / \b doubles \b /ix
-                    || $pod =~ / \b (?: a | the | long ) \s+ (?: double | NV ) \b /ix)
+                   || $pod =~ / \b (double | single) $dash_or_spaces precision \b /ix
+                   || $pod =~ / \b doubles \b /ix
+                   || $pod =~ / \b (?: a | the | long ) \s+ (?: double | NV ) \b /ix)
             {
                 $configs{$name}{'section'} =
-                                    'Floating point configuration values';
+                                    $floating_scn;
             }
             else {
                 # Above are the specific sections.  The rest go into a
@@ -1003,7 +1143,7 @@ sub parse_config_h {
                     }
                 }
 
-                $configs{$name}{'section'} = 'General Configuration';
+                $configs{$name}{'section'} = $genconfig_scn;
             }
 
             my $section = $configs{$name}{'section'};
@@ -1012,6 +1152,7 @@ sub parse_config_h {
             $flags .= 'AdmnT';
             $flags .= 'U' unless defined $configs{$name}{usage};
 
+            # All the information has been gathered; save it
             $docs{'api'}{$section}{$name}{flags} = $flags;
             $docs{'api'}{$section}{$name}{pod} = $configs{$name}{pod};
             $docs{'api'}{$section}{$name}{ret_type} = "";
@@ -1024,6 +1165,33 @@ sub parse_config_h {
     }
 }
 
+sub format_pod_indexes($) {
+    my $entries_ref = shift;
+
+    # Output the X<> references to the names, packed since they don't get
+    # displayed, but not too many per line so that when someone is editing the
+    # file, it doesn't run on
+
+    my $text ="";
+    my $line_length = 0;
+    for my $name (sort dictionary_order $entries_ref->@*) {
+        my $entry = "X<$name>";
+        my $entry_length = length $entry;
+
+        # Don't loop forever if we have a verrry long name, and don't go too
+        # far to the right.
+        if ($line_length > 0 && $line_length + $entry_length > $max_width) {
+            $text .= "\n";
+            $line_length = 0;
+        }
+
+        $text .= $entry;
+        $line_length += $entry_length;
+    }
+
+    return $text;
+}
+
 sub docout ($$$) { # output the docs for one function
     my($fh, $element_name, $docref) = @_;
 
@@ -1156,9 +1324,8 @@ sub docout ($$$) { # output the docs for one function
             my $name_indent = $indent + $longest_ret;
             $name_indent += $ret_name_sep_length if $longest_ret;
 
-            # 80 column terminal - 1 for pager adding a column; -7 for nroff
-            # indent;
-            my $max_length = 80 - 1 - 7 - $description_indent - $usage_indent;
+            my $this_max_width =
+                               $max_width - $description_indent - $usage_indent;
 
             for my $item (@items) {
                 my $ret_type = $item->{ret_type};
@@ -1213,7 +1380,7 @@ sub docout ($$$) { # output the docs for one function
                         #                                    short2,
                         #                                    very_long_argument,
                         #                                    short3)
-                        if ($total_length > $max_length) {
+                        if ($total_length > $this_max_width) {
 
                             # If this is the first continuation line,
                             # calculate the longest argument; this will be the
@@ -1242,9 +1409,10 @@ sub docout ($$$) { # output the docs for one function
                                 }
 
                                 # Calculate the new indent if necessary.
-                                $arg_indent = $max_length - $longest_arg_length
+                                $arg_indent =
+                                        $this_max_width - $longest_arg_length
                                         if $arg_indent + $longest_arg_length
-                                                                > $max_length;
+                                                            > $this_max_width;
                             }
 
                             print $fh "\n", (" " x $arg_indent);
@@ -1277,7 +1445,151 @@ sub docout ($$$) { # output the docs for one function
     print $fh "\n=for hackers\nFound in file $file\n";
 }
 
-sub sort_helper {
+sub construct_missings_section {
+    my ($pod_name, $missings_ref) = @_;
+    my $text = "";
+
+    return $text unless $missings_ref->@*;
+
+    $text .= <<~EOT;
+
+        =head1 $undocumented_scn
+
+        EOT
+    if ($pod_name eq 'perlapi') {
+        $text .= <<~'EOT';
+            The following functions have been flagged as part of the public
+            API, but are currently undocumented.  Use them at your own risk,
+            as the interfaces are subject to change.  Functions that are not
+            listed in this document are not intended for public use, and
+            should NOT be used under any circumstances.
+
+            If you feel you need to use one of these functions, first send
+            email to L<perl5-porters@perl.org|mailto:perl5-porters@perl.org>.
+            It may be that there is a good reason for the function not being
+            documented, and it should be removed from this list; or it may
+            just be that no one has gotten around to documenting it.  In the
+            latter case, you will be asked to submit a patch to document the
+            function.  Once your patch is accepted, it will indicate that the
+            interface is stable (unless it is explicitly marked otherwise) and
+            usable by you.
+            EOT
+    }
+    else {
+        $text .= <<~'EOT';
+            The following functions are currently undocumented.  If you use
+            one of them, you may wish to consider creating and submitting
+            documentation for it.
+            EOT
+    }
+
+    $text .= "\n" . format_pod_indexes($missings_ref);
+
+    # Sort the elements.
+    my @missings = sort dictionary_order $missings_ref->@*;
+
+
+    $text .= "\n";
+
+    use integer;
+
+    # Look through all the elements in the list and see how many columns we
+    # could place them in the output what will fit in the available width.
+    my $min_spacer = 2;     # Need this much space between columns
+    my $columns;
+    my $rows;
+    my @col_widths;
+
+  COLUMN:
+    # We start with more columns, and work down until we find a number that
+    # can accommodate all the data.  This algorithm doesn't require the
+    # resulting columns to all have the same width.  This can allow for
+    # as tight of packing as the data will possibly allow.
+    for ($columns = 7; $columns > 1; $columns--) {
+
+        # For this many columns, we will need this many rows (final row might
+        # not be completely filled)
+        $rows = (@missings + $columns - 1) / $columns;
+
+        my $row_width = 0;
+        my $i = 0;  # Which missing element
+
+        # For each column ...
+        for my $col (0 .. $columns - 1) {
+
+            # Calculate how wide the column needs to be, which is based on the
+            # widest element in it
+            $col_widths[$col] = 0;
+
+            # Look through all the rows to find the widest element
+            for my $row (0 .. $rows - 1) {
+
+                # Skip if this row doesn't have an entry for this column
+                last if $i >= @missings;
+
+                # This entry occupies this many bytes.
+                my $this_width = length $missings[$i];
+
+                # All but the final column need a spacer between it and the
+                # next column over.
+                $this_width += $min_spacer if $col < $columns - 1;
+
+
+                # This column will need to have enough width to accommodate
+                # this element
+                if ($this_width > $col_widths[$col]) {
+
+                    # We can't have this many columns if the total width
+                    # exceeds the available; bail now and try fewer columns
+                    next COLUMN if $row_width + $this_width > $max_width;
+
+                    $col_widths[$col] = $this_width;
+                }
+
+                $i++;   # The next row will contain the next item
+            }
+
+            $row_width += $col_widths[$col];
+            next COLUMN if $row_width > $max_width;
+        }
+
+        # If we get this far, this many columns works
+        last;
+    }
+
+    # Here, have calculated the number of rows ($rows) and columns ($columns)
+    # required to list the elements.  @col_widths contains the width of each
+    # column.
+
+    $text .= "\n\n=over $description_indent\n\n";
+
+    # Assemble the output
+    for my $row (0 .. $rows - 1) {
+        for my $col (0 .. $columns - 1) {
+            $text .= " " if $col == 0;  # Indent one to mark as verbatim
+
+            my $index = $row + $rows * $col;  # Convert 2 dimensions to 1
+
+            # Skip if this row doesn't have an entry for this column
+            next if $index >= @missings;
+
+            my $element = $missings[$index];
+            $text .= $element;
+
+            # Add alignment spaces for all but final column
+            $text .= " " x ($col_widths[$col] - length $element)
+                                                        if $col < $columns - 1;
+        }
+
+        $text .= "\n";  # End of row
+    }
+
+    $text .= "\n=back\n";
+
+    return $text;
+}
+
+sub dictionary_order {
     # Do a case-insensitive dictionary sort, with only alphabetics
     # significant, falling back to using everything for determinancy
     return (uc($a =~ s/[[:^alpha:]]//r) cmp uc($b =~ s/[[:^alpha:]]//r))
@@ -1286,7 +1598,7 @@ sub sort_helper {
 }
 
 sub output {
-    my ($podname, $header, $dochash, $missing, $footer) = @_;
+    my ($podname, $header, $dochash, $missings_ref, $footer) = @_;
     #
     # strip leading '|' from each line which had been used to hide
     # pod from pod checkers.
@@ -1298,7 +1610,7 @@ sub output {
 
     print $fh $header, "\n";
 
-    for my $section_name (sort sort_helper keys %valid_sections) {
+    for my $section_name (sort dictionary_order keys %valid_sections) {
         my $section_info = $dochash->{$section_name};
 
         # We allow empty sections in perlintern.
@@ -1321,9 +1633,8 @@ sub output {
             }
         }
 
-
-        if ($section_info) {
-            for my $function_name (sort sort_helper keys %$section_info) {
+        if ($section_info && keys $section_info->%*) {
+            for my $function_name (sort dictionary_order keys %$section_info) {
                 docout($fh, $function_name, $section_info->{$function_name});
             }
         }
@@ -1336,38 +1647,7 @@ sub output {
                             && defined $valid_sections{$section_name}{footer};
     }
 
-    if (@$missing) {
-        print $fh "\n=head1 Undocumented functions\n";
-        print $fh $podname eq 'perlapi' ? <<'_EOB_' : <<'_EOB_';
-
-The following functions have been flagged as part of the public API,
-but are currently undocumented.  Use them at your own risk, as the
-interfaces are subject to change.  Functions that are not listed in this
-document are not intended for public use, and should NOT be used under any
-circumstances.
-
-If you feel you need to use one of these functions, first send email to
-L<perl5-porters@perl.org|mailto:perl5-porters@perl.org>.  It may be
-that there is a good reason for the function not being documented, and it
-should be removed from this list; or it may just be that no one has gotten
-around to documenting it.  In the latter case, you will be asked to submit a
-patch to document the function.  Once your patch is accepted, it will indicate
-that the interface is stable (unless it is explicitly marked otherwise) and
-usable by you.
-
-_EOB_
-The following functions are currently undocumented.  If you use one of
-them, you may wish to consider creating and submitting documentation for
-it.
-
-_EOB_
-        print $fh "\n=over $description_indent\n";
-
-        for my $missing (sort sort_helper @$missing) {
-            print $fh "\n=item C<$missing>\nX<$missing>\n";
-        }
-        print $fh "\n=back\n";
-    }
+    print $fh construct_missings_section($podname, $missings_ref);
 
     print $fh "\n$footer\n=cut\n";
 
@@ -1422,19 +1702,24 @@ my @missing_api = grep $funcflags{$_}{flags} =~ /A/
                     && !$docs{api}{$_}, keys %funcflags;
 push @missing_api, keys %missing_macros;
 
-my $other_places = join ", ", map { "L<$_>" } sort sort_helper qw( perlclib perlxs),
-                                                               keys %described_elsewhere;
+my @other_places = ( qw(perlclib perlxs), keys %described_elsewhere );
+my $places_other_than_intern = join ", ",
+            map { "L<$_>" } sort dictionary_order 'perlapi', @other_places;
+my $places_other_than_api = join ", ",
+            map { "L<$_>" } sort dictionary_order 'perlintern', @other_places;
 
 # The S< > makes things less densely packed, hence more readable
-my $has_defs_text .= join ",S< > ", map { "C<$_>" } sort sort_helper @has_defs;
-my $has_r_defs_text .= join ",S< > ", map { "C<$_>" } sort sort_helper @has_r_defs;
-$valid_sections{'General Configuration'}{footer} =~ s/__HAS_LIST__/$has_defs_text/;
-$valid_sections{'General Configuration'}{footer} =~ s/__HAS_R_LIST__/$has_r_defs_text/;
+my $has_defs_text .= join ",S< > ", map { "C<$_>" } sort dictionary_order @has_defs;
+my $has_r_defs_text .= join ",S< > ", map { "C<$_>" } sort dictionary_order @has_r_defs;
+$valid_sections{$genconfig_scn}{footer} =~ s/__HAS_LIST__/$has_defs_text/;
+$valid_sections{$genconfig_scn}{footer} =~ s/__HAS_R_LIST__/$has_r_defs_text/;
 
-my $include_defs_text .= join ",S< > ", map { "C<$_>" } sort sort_helper @include_defs;
-$valid_sections{'General Configuration'}{footer} =~ s/__INCLUDE_LIST__/$include_defs_text/;
+my $include_defs_text .= join ",S< > ", map { "C<$_>" } sort dictionary_order @include_defs;
+$valid_sections{$genconfig_scn}{footer} =~ s/__INCLUDE_LIST__/$include_defs_text/;
 
-my $section_list = join "\n\n", map { "=item L</$_>" } sort sort_helper keys %valid_sections;
+my $section_list = join "\n\n", map { "=item L</$_>" }
+                                sort(dictionary_order keys %valid_sections),
+                                $undocumented_scn;  # Keep last
 
 output('perlapi', <<"_EOB_", $docs{api}, \@missing_api, <<"_EOE_");
 |=encoding UTF-8
@@ -1452,7 +1737,7 @@ output('perlapi', <<"_EOB_", $docs{api}, \@missing_api, <<"_EOE_");
 |L<perlintern> and F<config.h>, some items are listed here as being actually
 |documented in another pod.
 |
-|L<At the end|/Undocumented functions> is a list of functions which have yet
+|L<At the end|/$undocumented_scn> is a list of functions which have yet
 |to be documented.  Patches welcome!  The interfaces of these are subject to
 |change without notice.
 |
@@ -1537,7 +1822,7 @@ _EOB_
 |
 |=head1 SEE ALSO
 |
-|F<config.h>, L<perlintern>, $other_places
+|F<config.h>, $places_other_than_api
 _EOE_
 
 # List of non-static internal functions
@@ -1570,5 +1855,5 @@ _EOB_
 |
 |=head1 SEE ALSO
 |
-|F<config.h>, L<perlapi>, $other_places
+|F<config.h>, $places_other_than_intern
 _EOE_
