@@ -17,33 +17,17 @@ our @EXPORT_OK = qw(
   dualvar isdual isvstring looks_like_number openhandle readonly set_prototype
   tainted
 );
-our $VERSION    = "1.56";
+our $VERSION    = "1.60";
 $VERSION =~ tr/_//d;
 
 require List::Util; # List::Util loads the XS
 List::Util->VERSION( $VERSION ); # Ensure we got the right XS version (RT#100863)
 
-our @EXPORT_FAIL;
-
-unless (defined &weaken) {
-  push @EXPORT_FAIL, qw(weaken);
-}
-unless (defined &isweak) {
-  push @EXPORT_FAIL, qw(isweak isvstring);
-}
-unless (defined &isvstring) {
-  push @EXPORT_FAIL, qw(isvstring);
-}
-
+# populating @EXPORT_FAIL is done in the XS code
 sub export_fail {
-  if (grep { /^(?:weaken|isweak)$/ } @_ ) {
-    require Carp;
-    Carp::croak("Weak references are not implemented in the version of perl");
-  }
-
   if (grep { /^isvstring$/ } @_ ) {
     require Carp;
-    Carp::croak("Vstrings are not implemented in the version of perl");
+    Carp::croak("Vstrings are not implemented in this version of perl");
   }
 
   @_;
@@ -241,24 +225,24 @@ true.
     $dual = isdual($foo);               # true
 
 Note that a scalar can be made to have both string and numeric content through
-numeric operations:
+standard operations:
 
     $foo = "10";
     $dual = isdual($foo);               # false
     $bar = $foo + 0;
     $dual = isdual($foo);               # true
 
-Note that although C<$!> appears to be a dual-valued variable, it is
-actually implemented as a magical variable inside the interpreter:
+The C<$!> variable is commonly dual-valued, though it is also magical in other
+ways:
 
     $! = 1;
+    $dual = isdual($!);                 # true
     print("$!\n");                      # "Operation not permitted"
-    $dual = isdual($!);                 # false
 
-You can capture its numeric and string content using:
-
-    $err = dualvar $!, $!;
-    $dual = isdual($err);               # true
+B<CAUTION>: This function is not as useful as it may seem. Dualvars are not a
+distinct concept in Perl, but a standard internal construct of all scalar
+values. Almost any value could be considered as a dualvar by this function
+through the course of normal operations.
 
 =head2 isvstring
 
@@ -324,12 +308,7 @@ Module use may give one of the following errors during import.
 
 =over
 
-=item Weak references are not implemented in the version of perl
-
-The version of perl that you are using does not implement weak references, to
-use L</isweak> or L</weaken> you will need to use a newer release of perl.
-
-=item Vstrings are not implemented in the version of perl
+=item Vstrings are not implemented in this version of perl
 
 The version of perl that you are using does not implement Vstrings, to use
 L</isvstring> you will need to use a newer release of perl.
