@@ -204,11 +204,11 @@ Perl_pad_new(pTHX_ int flags)
         SAVECOMPPAD();
         if (! (flags & padnew_CLONE)) {
             SAVESPTR(PL_comppad_name);
-            save_strlen((STRLEN *)&PL_padix);
-            save_strlen((STRLEN *)&PL_constpadix);
-            save_strlen((STRLEN *)&PL_comppad_name_fill);
-            save_strlen((STRLEN *)&PL_min_intro_pending);
-            save_strlen((STRLEN *)&PL_max_intro_pending);
+            SAVESTRLEN(PL_padix);
+            SAVESTRLEN(PL_constpadix);
+            SAVESTRLEN(PL_comppad_name_fill);
+            SAVESTRLEN(PL_min_intro_pending);
+            SAVESTRLEN(PL_max_intro_pending);
             SAVEBOOL(PL_cv_has_eval);
             if (flags & padnew_SAVESUB) {
                 SAVEBOOL(PL_pad_reset_pending);
@@ -719,7 +719,7 @@ Perl_pad_alloc(pTHX_ I32 optype, U32 tmptype)
         pad_reset();
     if (tmptype == SVs_PADMY) { /* Not & because this ‘flag’ is 0.  */
         /* For a my, simply push a null SV onto the end of PL_comppad. */
-        sv = *av_fetch(PL_comppad, AvFILLp(PL_comppad) + 1, TRUE);
+        sv = *av_store_simple(PL_comppad, AvFILLp(PL_comppad) + 1, newSV(0));
         retval = (PADOFFSET)AvFILLp(PL_comppad);
     }
     else {
@@ -746,7 +746,7 @@ Perl_pad_alloc(pTHX_ I32 optype, U32 tmptype)
             if (++retval <= names_fill &&
                    (pn = names[retval]) && PadnamePV(pn))
                 continue;
-            sv = *av_fetch(PL_comppad, retval, TRUE);
+            sv = *av_fetch_simple(PL_comppad, retval, TRUE);
             if (!(SvFLAGS(sv) &
 #ifdef USE_PAD_RESET
                     (konst ? SVs_PADTMP : 0)
@@ -1402,17 +1402,17 @@ void
 Perl_pad_block_start(pTHX_ int full)
 {
     ASSERT_CURPAD_ACTIVE("pad_block_start");
-    save_strlen((STRLEN *)&PL_comppad_name_floor);
+    SAVESTRLEN(PL_comppad_name_floor);
     PL_comppad_name_floor = PadnamelistMAX(PL_comppad_name);
     if (full)
         PL_comppad_name_fill = PL_comppad_name_floor;
     if (PL_comppad_name_floor < 0)
         PL_comppad_name_floor = 0;
-    save_strlen((STRLEN *)&PL_min_intro_pending);
-    save_strlen((STRLEN *)&PL_max_intro_pending);
+    SAVESTRLEN(PL_min_intro_pending);
+    SAVESTRLEN(PL_max_intro_pending);
     PL_min_intro_pending = 0;
-    save_strlen((STRLEN *)&PL_comppad_name_fill);
-    save_strlen((STRLEN *)&PL_padix_floor);
+    SAVESTRLEN(PL_comppad_name_fill);
+    SAVESTRLEN(PL_padix_floor);
     /* PL_padix_floor is what PL_padix is reset to at the start of each
        statement, by pad_reset().  We set it when entering a new scope
        to keep things like this working:
